/*This is autogenerated file*/

#ifndef __C_ICAP_CONF_H
#define __C_ICAP_CONF_H


/*C_ICAP VERSION*/
#define C_ICAP_HEX_VERSION 0x00000005000D

/* Define USE_IPV6 if we are supporting ipv6 */
#if 0
#define USE_IPV6
#endif

#if 0
#define USE_OPENSSL
#endif

#if 1
#define USE_SYSV_IPC
#endif

#if  1
#define USE_POSIX_MAPPED_FILES
#endif

#if 0
#define USE_POSIX_SHARED_MEM
#endif

#if 1
#   define USE_SYSV_IPC_MUTEX
#endif
#if 1
#  define USE_POSIX_FILE_LOCK
#endif
#if 0
#  define USE_POSIX_SEMAPHORES
#endif

#if 0
#define USE_PTHREADS_RWLOCK
#endif

#if 1
#define USE_REGEX
#endif

#if 0
#define __CI_COMPAT
#endif

#if 1
#define USE_POLL
#endif

/*The following maybe should used...*/
#if 1
#define __SYS_TYPES_H_EXISTS
#endif

#if 1
#define __INTTYPES_H_EXISTS
#endif

/*
 The 64 bit data models as described by wikipedia are:
   Data model       short  int      long   long long      pointers/size_t
   LLP64             16    32       32       64                64
   LP64              16    32       64       64                64
   ILP64             16    64       64       64                64
   SILP64            64    64       64       64                64

 On most 32bit and 64bit unix machines, 'short' is always 16bit, 'int' is 32bit
 long is 32bit or 64bit respectively and long long is always 64bit.
 We are going to define:
      off_t as long on 32bit machines and long long on 64bit (sizeof(void*)==8)
      size_t as long on 32bit machines and long long on 64bit

  The c-icap should not use int32_t/uint32_t/int16_t/uint16_t types
*/

#define CI_SIZEOF_VOID_P 4

/* typedef off_t  if <sys/types.h> does not define. */
#if 0
#if CI_SIZEOF_VOID_P == 8
typedef long long off_t;
#define CI_SIZEOF_OFF_T 8
#elif CI_SIZEOF_VOID_P == 4
typedef long off_t;
#define CI_SIZEOF_OFF_T 4
#else
#error "Unhandled size of void *"
#endif
#else /*if 0*/
#define CI_SIZEOF_OFF_T 8
#endif

/* typedef size_t if <sys/types.h> does not define. */
#if 0
#if SIZEOF_VOID_P == 8
typedef unsigned long long size_t;
#elif SIZEOF_VOID_P == 4
typedef unsigned int size_t;
#else
#error "Unhandled size of void *"
#endif
#endif

#if 0
typedef unsigned char uint8_t;
#endif

#if 0
typedef char int8_t;
#endif

#if 0
typedef unsigned long long uint64_t;
#endif

#if 0
typedef long long int64_t;
#endif

#if 1
#define USE_VISIBILITY_ATTRIBUTE
#endif

#endif
